using System.Collections.ObjectModel;
using Zeroconf;

namespace BindingBug.App.Presentation.DeviceConnection;

public partial class ConnectionEthernetViewModel : ObservableObject
{
    private readonly IStringLocalizer _localizer;
    
    [ObservableProperty][NotifyPropertyChangedFor(nameof(CanScan))] private bool _isScanning;
    public bool CanScan => !IsScanning && !IsConnecting;
    [ObservableProperty] private string? _status;
    [ObservableProperty] private bool _hasDevices;
    [ObservableProperty] private ObservableCollection<string> _chargers = new();
    [ObservableProperty] private string? _selectedCharger;
    [ObservableProperty] private string? _connectingStatus;
    [ObservableProperty][NotifyPropertyChangedFor((nameof(CanScan)))] private bool _isConnecting;

    public ConnectionEthernetViewModel(IStringLocalizer localizer, IOptions<AppConfig> appInfo, INavigator navigator)
    {
        _localizer = localizer;
        Status = localizer.GetString("ConnectionEthernet_NoDeviceDiscovered");
    }
    
    [RelayCommand]
    public async Task Scan()
    {
        IsScanning = true;
        Status = _localizer.GetString("ConnectionEthernet_Scanning");

        await Task.Delay(5000);
        
        Chargers.Clear();

        ILookup<string, string> domains = await ZeroconfResolver.BrowseDomainsAsync();
        var responses = await ZeroconfResolver.ResolveAsync(domains.Select(g => g.Key));
        
        foreach (var response in responses)
        {
            Chargers.Add(response.DisplayName);
            //System.Diagnostics.Debug.WriteLine(response);
        }
        
        if (Chargers.Any())
        {
            SelectedCharger = Chargers[0];
        }
        
        Status = Chargers.Count switch
        {
            0 => _localizer.GetString("ConnectionEthernet_NoDeviceDiscovered"),
            1 => _localizer.GetString("ConnectionEthernet_SingularDeviceDetected"),
            _ => string.Format(_localizer.GetString("ConnectionEthernet_PluralDeviceDetected"), Chargers.Count)
        };
        
        HasDevices = Chargers.Any();
        IsScanning = false;
    }
    
    [RelayCommand]
    public async Task Connect()
    {
        IsConnecting = true;
        
        ConnectingStatus = string.Format(_localizer.GetString("ConnectionEthernet_ConnectingTo"), SelectedCharger!.Substring(0, Math.Min(16, SelectedCharger.Length - 1)));
        await Task.Delay(5000);

        IsConnecting = false;
    }
    
    [RelayCommand]
    public void Cancel()
    {
        IsConnecting = false;
    }
}
